@@ -161,7 +161,7 @@ sub invoke {
     eval {
 	$cmd = get_cmd ($pkg, $cmd, $xd);
 	$cmd->{svnconfig} = $xd->{svnconfig} if $xd;
-	$cmd->getopt (\@args, 'h|help|?' => \$help);
+	$cmd->getopt (\@args, 'h|help|?' => \$help, 'encoding=s' => \$cmd->{encoding});
 	$cmd->_subcommand;
 
 	# Fake shell globbing on Win32 if we are called from main
@@ -173,6 +173,9 @@ sub invoke {
 	    } @args;
 	}
 
+	# XXX: xd needs to know encoding too
+	$xd->{encoding} = $cmd->{encoding}
+	    if $xd;
 	if ($help || !(@args = $cmd->parse_arg(@args))) {
 	    select STDERR unless $output;
 	    $cmd->usage;
@@ -688,9 +691,7 @@ could be given to extract the usage from the POD.
 
 sub brief_usage {
     my ($self, $file) = @_;
-    my $fname = ref($self);
-    $fname =~ s|::|/|g;
-    open my ($podfh), '<', ($file || $INC{"$fname.pm"}) or return;
+    open my ($podfh), '<', ($file || $self->filename) or return;
     local $/=undef;
     my $buf = <$podfh>;
     if($buf =~ /^=head1\s+NAME\s*SVK::Command::(\w+ - .+)$/m) {
@@ -699,6 +700,20 @@ sub brief_usage {
     close $podfh;
 }
 
+=head3 filename
+
+Return the filename for the command module.
+
+=cut
+
+sub filename {
+    my $self = shift;
+    my $fname = ref($self);
+    $fname =~ s{::[a-z]+}{}; # subcommand
+    $fname =~ s{::}{/}g;
+    $INC{"$fname.pm"}
+}
+
 =head3 usage ($want_detail)
 
 Display usage.  If C<$want_detail> is true, the C<DESCRIPTION>
@@ -708,16 +723,12 @@ section is displayed as well.
 
 sub usage {
     my ($self, $want_detail) = @_;
-    # XXX: the order from selected is not preserved.
-    my $fname = ref($self);
-    $fname =~ s|::|/|g;
-
-    my($cmd) = $fname =~ m{\W(\w+)$};
-
+    my $fname = $self->filename;
+    my($cmd) = $fname =~ m{\W(\w+)\.pm$};
     my $parser = Pod::Simple::Text->new;
     my $buf;
     $parser->output_string(\$buf);
-    $parser->parse_file($INC{"$fname.pm"});
+    $parser->parse_file($fname);
 
     $buf =~ s/SVK::Command::(\w+)/\l$1/g;
     $buf =~ s/^AUTHORS.*//sm;
