@@ -5,18 +5,13 @@ use base qw( SVK::Command::Copy );
 use SVK::Util qw ( abs2rel );
 use SVK::I18N;
 
-sub lock {
-    my $self = shift;
-    $self->lock_coroot(@_);
-}
-
 sub handle_direct_item {
     my $self = shift;
     my ($editor, $anchor, $m, $src, $dst) = @_;
-    my ($srcm) = $self->under_mirror ($src);
+    my $srcm = $self->under_mirror ($src);
     my $call;
-    if ($srcm && $srcm->path eq $src->path) {
-	# XXX: this should be in svk::mirror
+    if ($srcm && $srcm->{target_path} eq $src->path) {
+	# this should be in svn::mirror
 	my $props = $src->root->node_proplist($src->path);
 	# this is very annoying: for inejecting an additional
 	# editor call, has to give callback to Command::Copy's
@@ -31,15 +26,15 @@ sub handle_direct_item {
     $self->SUPER::handle_direct_item (@_, $call);
 
     $editor->delete_entry (abs2rel ($src->path, $anchor => undef, '/'),
-			   $m ? scalar $m->find_remote_rev ($src->revision)
-			      : $src->revision, 0);
+			   $m ? scalar $m->find_remote_rev ($src->{revision})
+			      : $src->{revision}, 0);
     $self->adjust_anchor ($editor);
 }
 
 sub handle_co_item {
     my ($self, $src, $dst) = @_;
     $self->SUPER::handle_co_item ($src->new, $dst); # might be modified
-    $self->{xd}->do_delete($src);
+    $self->{xd}->do_delete (%$src);
 }
 
 sub run {
@@ -48,7 +43,7 @@ sub run {
     my $ret = $self->SUPER::run(@_);
     if ($self->{post_process_mirror}) {
 	# XXX: also should set svm:incomplete revprop
-	# should be in SVK::Mirror as well
+	# should be in SVN::Mirror as well
 	my $mstring = $src->root->node_prop('/', 'svm:mirror');
 	for (@{$self->{post_process_mirror}}) {
 	    my ($from, $to) = @$_;
@@ -56,7 +51,8 @@ sub run {
 	}
 	my $cmd = $self->command('propset', { revision => undef,
 					      message => 'svk: fix-up for mirror move' });
-	$cmd->run('svm:mirror', $mstring, $src->new(path => '/'));
+	$cmd->run($cmd->parse_arg('svm:mirror', $mstring,
+				  '/'.$src->depotname.'/'));
     }
     return $ret;
 }
@@ -74,8 +70,6 @@ SVK::Command::Move - Move a file or directory
 =head1 OPTIONS
 
  -r [--revision] REV    : act on revision REV instead of the head revision
- -p [--parent]          : create intermediate directories as required
- -q [--quiet]           : print as little as possible
  -m [--message] MESSAGE : specify commit message MESSAGE
  -F [--file] FILENAME   : read commit message from FILENAME
  --template             : use the specified message as the template to edit
