@@ -38,8 +38,15 @@ SVK::Command::Propdel - Delete a property on files or dirs
 =head1 OPTIONS
 
  -R [--recursive]       : descend recursively
- -r [--revision] REV	: act on revision REV instead of the head revision
+ -r [--revision] REV    : act on revision REV instead of the head revision
  --revprop              : operate on a revision property (use with -r)
+ -m [--message] MESSAGE : specify commit message MESSAGE
+ -F [--file] FILENAME   : read commit message from FILENAME
+ --template             : use the specified message as the template to edit
+ --encoding ENC         : treat -m/-F value as being in charset encoding ENC
+ -P [--patch] NAME      : instead of commit, save this change as a patch
+ -S [--sign]            : sign this change
+ -C [--check-only]      : try operation but make no changes
  --direct               : commit directly even if the path is mirrored
 
 =head1 AUTHORS
