@@ -11,7 +11,6 @@ sub options {
     ($_[0]->SUPER::options,
      'r|revision=i' => 'rev',
      'revprop' => 'revprop',
-     'q|quiet' => 'quiet',
     );
 }
 
@@ -25,26 +24,18 @@ sub parse_arg {
 sub lock {
     my $self = shift;
     my @paths = @_[2..$#_];
-    return unless grep {$_->isa('SVK::Path::Checkout')} @paths;
-    $self->lock_target ($self->{xd}->target_condensed(@_[2..$#_]));
+    return unless grep {$_->copath} @paths;
+    $self->lock_target ($self->arg_condensed (map {$_->{report}} @_[2..$#_]));
 }
 
 sub do_propset_direct {
     my ($self, $target, $propname, $propvalue) = @_;
 
     if ($self->{revprop}) {
-	my $fs = $target->repos->fs;
-        my $rev = (defined($self->{rev}) ? $self->{rev} : $target->revision);
+	my $fs = $target->{repos}->fs;
+        my $rev = (defined($self->{rev}) ? $self->{rev} : $target->{revision});
         $fs->change_rev_prop ($rev, $propname => $propvalue);
-	unless ($self->{quiet}) {
-	    if (defined $propvalue) {
-		print loc("Property '%1' set on repository revision %2.\n",
-		    $propname, $rev);
-	    } else {
-		print loc("Property '%1' deleted from repository revision %2.\n",
-		    $propname, $rev);
-	    }
-	}
+        print loc("Property '%1' set on repository revision %2.\n", $propname, $rev);
         return;
     }
 
@@ -59,7 +50,7 @@ sub do_propset_direct {
     my $path = abs2rel ($target->path, $anchor => undef, '/');
 
     my $m = $self->under_mirror ($target);
-    my $rev = $target->revision;
+    my $rev = $target->{revision};
     $rev = $m->find_remote_rev ($rev) if $m;
     if ($kind == $SVN::Node::dir) {
 	if ($anchor eq $target->path) {
@@ -85,19 +76,21 @@ sub do_propset_direct {
 sub do_propset {
     my ($self, $pname, $pvalue, $target) = @_;
 
-    if ($target->isa('SVK::Path::Checkout')) {
+    if ($target->{copath}) {
 	die loc("-r not allowed for propset copath.\n")
 	    if $self->{rev};
 	# verify the content is not with mixed line endings.
 	if ($pname eq 'svn:eol-style') {
-	    my $fh = $target->root->file_contents($target->path);
-	    binmode($fh, SVK::XD::get_eol_layer({ 'svn:eol-style' => $pvalue }, '<', 1));
+	    my $xdroot = $target->root ($self->{xd});
+	    my $fh = SVK::XD::get_fh ($xdroot, '<', $target->{path}, $target->{copath},
+				      { 'svn:eol-style' => $pvalue }, '',
+				      undef, 1);
 	    eval {
 		local $/ = \16384;
 		while (<$fh>) { };
-	    } if $fh;
+	    };
 	    if ($@ =~ m/Mixed/) {
-		die loc ("File %1 has inconsistent newlines.\n", $target->report);
+		die loc ("File %1 has inconsistent newlines.\n", $target->{report});
 	    }
 	    elsif ($@) {
 		die $@;
@@ -105,10 +98,9 @@ sub do_propset {
 	}
 
 	$self->{xd}->do_propset
-	    ( $target->for_checkout_delta,
+	    ( %$target,
 	      propname => $pname,
 	      propvalue => $pvalue,
-	      quiet => $self->{quiet},
 	    );
     }
     else {
@@ -150,7 +142,6 @@ SVK::Command::Propset - Set a property on path
  -P [--patch] NAME      : instead of commit, save this change as a patch
  -S [--sign]            : sign this change
  -C [--check-only]      : try operation but make no changes
- -q [--quiet]           : print as little as possible
  --direct               : commit directly even if the path is mirrored
 
 =head1 AUTHORS
